/***********************************************************************
**
**   ipc.h
**
**   This file is part of Cumulus
**
************************************************************************
**
**   Copyright (c):  2004 by Axel Pauli (axel@kflog.org)
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 2 of the License, or
**   (at your option) any later version.
**
**   $Id: ipc.h,v 1.2 2004/04/04 12:25:41 eckhard Exp $
**
***********************************************************************/

#ifndef _Ipc_hh_
#define _Ipc_hh_ 1

#include <qstring.h>

/**
 * This class manages the low layer interfaces for the interprocess
 * cummunication via sockets. The server part can handle up to two client
 * connections. All io is done in blocking mode.
 */

#define IPC_IP "127.0.0.1"

class Ipc
{
  //----------------------------------------------------------------------
  // Public data elements of Class
  //----------------------------------------------------------------------

public:

  static const char * const moduleName;

  //----------------------------------------------------------------------
  // Private data elements of Class
  //----------------------------------------------------------------------

public:

  // Constructor
  Ipc();

  // Destructor
  ~Ipc();

  //----------------------------------------------------------------------
  // Server class definition
  //----------------------------------------------------------------------

  class Server
  {
  private:

    QString host;
    QString ipAddress;
    ushort  listenPort;

    int listenSock;
    int clientSocks[2];
    
  public:

    Server();

    ~Server();

    //----------------------------------------------------------------------
    // initialize server, a listening endpoint is created.
    //
    // IN ipAddress: ip address used in bind call, if empty or null
    //               INADDR_ANY is used.
    // IN port:      port number to be used in bind call, if 0, kernel
    //               will assign a free number.
    // Returns:      true on success otherwise false
    //----------------------------------------------------------------------
    const bool init( const char *ipAddress=0,
		     const unsigned short port=0 );

    //----------------------------------------------------------------------
    // Returns the socket descriptor of the next available client connection
    // or -1 in error case.
    //----------------------------------------------------------------------
    
    const int connect2Client(uint index);

    //----------------------------------------------------------------------
    // Closes the client connection.
    // Returns -1 in error case otherwise 0
    //----------------------------------------------------------------------
 
    const int closeClientSock(uint index);

    //----------------------------------------------------------------------
    // Closes the listen socket.
    // Returns -1 in error case otherwise 0
    //----------------------------------------------------------------------
 
    const int closeListenSock();

    //----------------------------------------------------------------------
    // Reads one time data from the connected client socket and returns.
    // Returns -1 in error case or number of read bytes
    //----------------------------------------------------------------------
    
    const int readMsg( uint index, void *data, int length );

    //----------------------------------------------------------------------
    // Writes the passed data to the connected client socket
    // Returns -1 in error case or number of written bytes
    //----------------------------------------------------------------------
    
    const int writeMsg( uint index, void *data, int length );

    const int getListenSock() const
    {
      return listenSock;
    };

    const int getClientSock(uint index) const
    {
      if( index == 0 )
	return clientSocks[0];
      else
	return clientSocks[1];
    };

    const unsigned short getListenPort() const
    {
      return listenPort;
    };

    const char *getIpAddress()
    {
      return ipAddress.data();
    };

  };

  //----------------------------------------------------------------------
  // Client class definition
  //----------------------------------------------------------------------

  public:

  class Client
  {
  private:

    QString host;
    QString ipAddress;
    unsigned short port;
    int sock;
    
  public:

    Client();

    ~Client();

    //----------------------------------------------------------------------
    // establishes a connection to the server
    //
    // IN ipAddress: ip address used in connect call, if empty or null
    //               address of local host (127.0.0.1) is used.
    // IN port:      port number to be used in connect call
    //
    // Returns: -1 in error case otherwise 0
    //----------------------------------------------------------------------

    const int connect2Server( const char *ipAddressIn,
			      const unsigned short portIn );

    //----------------------------------------------------------------------
    // Reads one time data from the connected server socket and returns.
    // Returns -1 in error case or number of read bytes
    //----------------------------------------------------------------------
    
    const int readMsg( void *data, int length );

    //----------------------------------------------------------------------
    // Writes the passed data to the connected server socket
    // Returns -1 in error case or number of written bytes
    //----------------------------------------------------------------------
    
    const int writeMsg( void *data, int length );

    //----------------------------------------------------------------------
    // Closes the socket of the server connection
    // Returns -1 in error case otherwise 0
    //----------------------------------------------------------------------

    const int closeSock();

    const int getSock() const
    {
      return sock;
    };
    
    const unsigned short getPort() const
    {
      return port;
    };

    const char *getIpAddress()
    {
      return ipAddress.data();
    };

  };

};

#endif  // #ifndef _Ipc_hh_
